"""
BaseExtractor is the abstract base class for zip extractors classes.

It defines some basic stuff like checking if zip exists, opening the zip,
 getting its contents, extracting a list of files from it, and closing the zip file.
"""

from abc import ABC, abstractmethod
from pathlib import Path
from typing import List, Union
from zipfile import ZipFile

from remotezip import RemoteZip


class BaseExtractor(ABC):
    """BaseExtractor provides abstract methods for dealing with zip files."""

    file: Union[ZipFile, RemoteZip]
    files: List[str]
    _out_dir: Path

    def __init__(self, _out_dir):
        """
        Initialize BaseExtractor class.

        :param _out_dir: output directory to place the extracted zip in.
        """
        self._out_dir = _out_dir
        self.files = []

    @abstractmethod
    def exists(self):
        """
        Check if the zip file exists.

        :return: should return True if file exists, False otherwise.
        """
        raise NotImplementedError

    @abstractmethod
    def open(self):
        """
        Open a remote zip to use afterwards.

        :return:
        """
        raise NotImplementedError

    @abstractmethod
    def get_files_list(self):
        """
        Get contents of the remote zip file as list and store it in files attribute.

        :return:
        """
        raise NotImplementedError

    def extract(self, files_to_extract: List[str]):
        """
        Extract a list of files from the zip file into an output directory.

        :param files_to_extract: a list of files to extract
        :return:
        """
        self.file.extractall(path=self._out_dir, members=files_to_extract)

    def close(self):
        """
        Close the zip file.

        :return:
        """
        self.file.close()

    @abstractmethod
    def get_file_name(self) -> str:
        """
        Get input zip file name and return it as a string.

        :return: a string of the input zip file name.
        """
        raise NotImplementedError



"""Miscellaneous functions used by the tool."""
import re


def extract_codename(updater_script) -> str:
    r"""Extract device codename form updater-script.

    Regex pattern explanation:
    - (?:/[\w\d_-]+/([\w\d]+):\d) matches codename in update fingerprint like
    Xiaomi/cmi_global/cmi:11/RKQ1.200710.002/V12.1.2.0.RJAMIXM:user/release-keys
    - (?:\(\"ro\.product\.device\"\) == \"([\w\d]+)\") matches codename
     in getprop("ro.product.device") statements
    - (?:get_device_compatible\(\"([\w\d]+)\"\)) matches get_device_compatible() statements
    :param updater_script: updater-script file as string
    :return: extracted codename if found or 'codename' if not found.
    """
    pattern = re.compile(r'(?:/[\w\d_-]+/([\w\d]+):\d)|'
                         r'(?:\(\"ro\.product\.device\"\) == \"([\w\d]+)\")|'
                         r'(?:get_device_compatible\(\"([\w\d]+)\"\))')
    match = pattern.search(updater_script)
    if match:
        codename = [i for i in match.groups() if i is not None]
        return codename[0]
    return 'codename'



"""Mock a zip file and create a placeholder files of its contents."""
from pathlib import Path
from shutil import make_archive, rmtree
from sys import argv
from typing import Union
from zipfile import ZipFile


def mock_zip(zip_file: str, out_dir: Union[str, Path]):
    """Create a mock zip of a given zip file.

    It iterates over the zip contents, creates the folders
    and creates a file that contains one byte of every file inside the zip
    :param out_dir: zip file to mock
    :param out_dir: out directory
    :type zip_file: str
    :type out_dir: str or Path
    """
    if isinstance(out_dir, str):
        out_dir = Path(out_dir) / 'mock_tmp'
    else:
        out_dir = out_dir / 'mock_tmp'
    out_dir = out_dir.absolute()
    out_dir.mkdir(parents=True, exist_ok=True)
    zip_file = Path(zip_file)

    with ZipFile(zip_file, 'r') as zipfile:
        files = zipfile.namelist()
        for item in files:
            item_path = Path(out_dir / item)
            if item.endswith('/'):
                if not item_path.exists():
                    item_path.mkdir(parents=True, exist_ok=True)
            else:
                if not item_path.exists():
                    item_path.parent.mkdir(parents=True, exist_ok=True)
                if item.endswith("updater-script"):
                    zipfile.extract(item, out_dir)
                else:
                    with open(f"{out_dir}/{item}", 'wb') as out:
                        out.write(b'')

    mocked_zip = f"{out_dir.parent}/mocked_{zip_file.stem}"
    make_archive(mocked_zip, zip_file.suffix.split('.')[-1], out_dir)
    out = Path(f"{mocked_zip}.zip")
    if not out.exists():
        raise RuntimeError("Could not create mocked zip file!")
    rmtree(out_dir)


if __name__ == '__main__':
    mock_zip(argv[1], argv[2])



"""Remote Zip Extractor is a concrete class that implements BaseExtractor methods \
that deals with remote zip files using remotezip library."""

from remotezip import RemoteZip
from requests import head

from xiaomi_flashable_firmware_creator.extractors.base_extractor import BaseExtractor


class RemoteZipExtractor(BaseExtractor):
    """RemoteZipExtractor provides methods for dealing with remote zip files."""

    zip_url: str
    zip_file: RemoteZip

    def __init__(self, zip_url: str, out_dir):
        """
        Initialize RemoteZipExtractor.

        :param zip_url: a direct url to a zip that contains a full recovery ROM.
        :param out_dir: output directory to place the extracted zip in.
        """
        self.zip_url = zip_url
        super().__init__(out_dir)

    def exists(self) -> bool:
        """
        Check if the remote file exists.

        :return: True if http code is 200 OK, False otherwise.
        """
        return head(self.zip_url).ok

    def open(self):
        """
        Open a remote zip to use afterwards.

        :return:
        """
        self.file = RemoteZip(self.zip_url)

    def get_files_list(self):
        """
        Get contents of the remote zip file as list and store it in files attribute.

        :return:
        """
        self.files = self.file.namelist()

    def get_file_name(self) -> str:
        """
        Get input zip url file name and return it as a string.

        :return: a string of the input zip url file name.
        """
        return self.zip_url.split('/')[-1]



#!/usr/bin/env python3
"""Xiaomi Flashable Firmware Creator cli module."""

from argparse import ArgumentParser

from xiaomi_flashable_firmware_creator.firmware_creator import FlashableFirmwareCreator


def arg_parse() -> (str, str, str):
    """
    Parse command-line arguments.

    This function parse command-line arguments and return
     the required parameters to be used in FlashableFirmwareCreator class constructor.

    :return: file, process, output dir
    """
    output = None
    parser = ArgumentParser(prog='python3 -m xiaomi_flashable_firmware_creator',
                            description='Xiaomi Flashable Firmware Creator')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("-F", "--firmware", help="Create normal Firmware zip")
    group.add_argument("-N", "--nonarb", help="Create non-ARB Firmware zip")
    group.add_argument("-L", "--firmwareless", help="Create Firmware-less zip")
    group.add_argument("-V", "--vendor", help="Create Firmware+Vendor zip")
    parser.add_argument('-o', "--output", help="Output directory")
    args = parser.parse_args()
    if args.output:
        output = args.output
        del args.output
    choose = {k: v for k, v in vars(args).items() if v is not None}
    process, zip_ = list(choose.items())[0]
    return zip_, process, output


def main():
    """Xiaomi Flashable Firmware Creator main module."""
    zip_, process, output = arg_parse()
    firmware_creator = FlashableFirmwareCreator(zip_, process, output)
    print(f"Unzipping MIUI... ({firmware_creator.type.name}) device")
    new_zip = firmware_creator.auto()
    print(f"All done! Output file is {new_zip}")



"""Local Zip Extractor is a concrete class that implements BaseExtractor methods \
that deals with local zip files using zipfile standard library."""

from pathlib import Path
from typing import Union
from zipfile import ZipFile

from xiaomi_flashable_firmware_creator.extractors.base_extractor import BaseExtractor


class LocalZipExtractor(BaseExtractor):
    """LocalZipExtractor provides methods for dealing with local zip files."""

    zip_file_path: Union[Path, str]
    zip_file: ZipFile

    def __init__(self, zip_file: Union[str, Path], out_dir):
        """
        Initialize LocalZipExtractor.

        :param zip_file: a path object or a string to a zip that contains a full recovery ROM.
        :param out_dir: output directory to place the extracted zip in.
        """
        self.zip_file_path = Path(zip_file) if isinstance(
            zip_file, str) else zip_file
        super().__init__(out_dir)

    def exists(self):
        """
        Check if the local zip file exists.

        :return: True if zip file exists, False otherwise.
        """
        return self.zip_file_path.exists()

    def open(self):
        """
        Open a local zip to use afterwards.

        :return:
        """
        self.file = ZipFile(self.zip_file_path, 'r')

    def get_files_list(self):
        """
        Get contents of the local zip file as list and store it in files attribute.

        :return:
        """
        self.files = self.file.namelist()

    def get_file_name(self):
        """
        Get input zip file name and return it as a string.

        :return: a string of the input zip file name.
        """
        return self.zip_file_path.name



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xiaomi_flashable_firmware_creator',
 'xiaomi_flashable_firmware_creator.extractors',
 'xiaomi_flashable_firmware_creator.helpers']

package_data = \
{'': ['*'], 'xiaomi_flashable_firmware_creator': ['templates/*']}

install_requires = \
['remotezip>=0.9.2,<0.10.0']

entry_points = \
{'console_scripts': ['xiaomi_flashable_firmware_creator = '
                     'xiaomi_flashable_firmware_creator.xiaomi_flashable_firmware_creator:main']}

setup_kwargs = {
    'name': 'xiaomi-flashable-firmware-creator',
    'version': '2.0.0',
    'description': 'Create flashable firmware zip from MIUI Recovery ROMs!',
    'long_description': '## Xiaomi Flashable Firmware Creator\n\nCreate flashable firmware zip from MIUI Recovery ROMs!\n\n[![PyPI version](https://badge.fury.io/py/xiaomi-flashable-firmware-creator.svg)](https://pypi.org/project/xiaomi-flashable-firmware-creator/)\n[![made-with-python](https://img.shields.io/badge/Made%20with-Python%203-3776AB?style=flat\\&labelColor=3776AB\\&logo=python\\&logoColor=white\\&link=https://www.python.org/)](https://www.python.org/)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/9c1f6cee01b74ef8a2fd0f0c787596a8)](https://www.codacy.com/gh/XiaomiFirmwareUpdater/xiaomi-flashable-firmware-creator.py/dashboard?utm_source=github.com\\&utm_medium=referral\\&utm_content=XiaomiFirmwareUpdater/xiaomi-flashable-firmware-creator.py\\&utm_campaign=Badge_Grade)\n[![Open Source Love](https://badges.frapsoft.com/os/v3/open-source.svg?v=103)](#) <br />\n[![PayPal](https://img.shields.io/badge/PayPal-Donate-00457C?style=flat\\&labelColor=00457C\\&logo=PayPal\\&logoColor=white\\&link=https://www.paypal.me/yshalsager)](https://www.paypal.me/yshalsager)\n[![Patreon](https://img.shields.io/badge/Patreon-Support-F96854?style=flat\\&labelColor=F96854\\&logo=Patreon\\&logoColor=white\\&link=https://www.paypal.me/yshalsager)](https://www.paypal.me/yshalsager)\n[![Liberapay](https://img.shields.io/badge/Liberapay-Support-F6C915?style=flat\\&labelColor=F6C915\\&logo=Liberapay\\&logoColor=white\\&link=https://liberapay.com/yshalsager)](https://liberapay.com/yshalsager)\n\nXiaomi Flashable Firmware Creator is a tool that generates flashable firmware-update packages from official MIUI ROMS.\n\nIt supports creating untouched firmware, non-arb firmware, firmware + vendor flashable zip, and firmware-less ROMs from any local zip file or direct link of the zip file.\n\n### Installation\n\nYou can simply install this tool using Python pip.\n\n```shell script\npip install xiaomi_flashable_firmware_creator\n```\n\n### CLI Usage\n\n```shell script\npython3 -m xiaomi_flashable_firmware_creator [-h] (-F FIRMWARE | -N NONARB | -L FIRMWARELESS | -V VENDOR) [-o OUTPUT]\n```\n\n**Examples:**\n\n*   Creating normal (untouched) firmware:\n\n```shell script\npython3 -m xiaomi_flashable_firmware_creator -F [MIUI ZIP]\n```\n\n*   Creating non-arb firmware (without anti-rollback):\n\n```shell script\npython3 -m xiaomi_flashable_firmware_creator -N [MIUI ZIP]\n```\n\n*   Creating firmware-less ROM (stock untouched ROM with just firmware removed):\n\n```shell script\npython3 -m xiaomi_flashable_firmware_creator -L [MIUI ZIP]\n```\n\n*   Creating firmware + vendor flashable zip:\n\n```shell script\npython3 -m xiaomi_flashable_firmware_creator -V [MIUI ZIP]\n```\n\n### Using from other Python scripts\n\n```python\nfrom xiaomi_flashable_firmware_creator.firmware_creator import FlashableFirmwareCreator\n\n# initialize firmware creator object with the following parameters:\n# input_file: zip file to extract from. It can be a local path or a remote direct url.\n# process: Which mode should the tool use. This must be one of "firmware", "nonarb", "firmwareless" or "vendor".\n# out_dir: The output directory to store the extracted file in.\n\nfirmware_creator = FlashableFirmwareCreator(input_zip, process, output_dir)\n# Now, you can either use auto() method to create the new zip file or do stuff at your own using firmware_creator public methods.\nnew_zip = firmware_creator.auto()\n```',
    'author': 'yshalsager',
    'author_email': 'ysh-alsager@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://xiaomifirmwareupdater.com/projects/xiaomi-flashable-firmware-creator/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
Xiaomi Flashable Firmware Creator class.

Xiaomi Flashable Firmware Creator class provides an easy way to
   create a flashable firmware zip file from any supported ROM.
Using extract, generate_updater_script, and make_zip you can create
   flashable files according to the provided configuration
   ('firmware', 'nonarb', 'firmwareless', 'vendor) .
"""

# pylint: disable=too-many-instance-attributes
from datetime import datetime
from pathlib import Path
from shutil import rmtree, make_archive
from socket import gethostname
from string import Template
from typing import Union, List

from xiaomi_flashable_firmware_creator import work_dir
from xiaomi_flashable_firmware_creator.extractors.local_zip_extractor import LocalZipExtractor
from xiaomi_flashable_firmware_creator.extractors.remote_zip_extractor import RemoteZipExtractor
from xiaomi_flashable_firmware_creator.helpers.misc import extract_codename
from xiaomi_flashable_firmware_creator.types import ProcessTypes, ZipTypes


class FlashableFirmwareCreator:
    """FlashableFirmwareCreator provides methods for creating \
     flashable firmware files from Xiaomi supported ROMs."""

    extractor: Union[LocalZipExtractor, RemoteZipExtractor]
    input_file: Union[str, Path]
    _tmp_dir: Path
    _updater_script_dir: Path
    host: str
    datetime: datetime
    _extract_mode: str
    extract_mode: ProcessTypes
    type: ZipTypes
    update_script: str

    def __init__(self, input_file, _extract_mode, out_dir=""):
        """
        Initialize FlashableFirmwareCreator.

        :param input_file: zip file to extract from.
         It can be a local path or a remote direct url.
        :param _extract_mode: Which mode should the tool use.
         This must be one of "firmware", "nonarb", "firmwareless" or "vendor".
        :param out_dir: The output directory to store the extracted file in.
        """
        self.input_file = input_file
        self._tmp_dir = Path(out_dir) / 'tmp' if out_dir else work_dir / 'tmp'
        self._out_dir = self._tmp_dir.parent.absolute()
        self._updater_script_dir = self._tmp_dir.absolute() / 'META-INF/com/google/android'
        self.host = gethostname()
        self.datetime = datetime.now()
        self.extract_mode = self.get_extract_mode(_extract_mode)
        self.update_script = ''
        self.extractor = self.get_extractor()
        self.init()

    def init(self):
        """
        Do some initial checks and housekeeping.

        - This method checks if the temporary directory exists
         and removes it then creates a new one.
        - Creates updater-script temporary path.
        - Checks if the input zip file exists and
         raise a FileNotFoundError exception if not.
        - Open the zip and get contents list.
        - Checks if the zip file is a valid ROM. If true get the ROM type.
         Otherwise, remove the temporary directory and raise a RuntimeError.
        """
        if self._tmp_dir.exists():
            rmtree(self._tmp_dir)
        self._tmp_dir.mkdir(parents=True, exist_ok=True)
        self._updater_script_dir.mkdir(parents=True, exist_ok=True)
        if not self.extractor.exists():
            raise FileNotFoundError(
                f"input file {self.input_file} does not exist!")
        self.extractor.open()
        self.extractor.get_files_list()
        if self.is_valid_rom():
            self.get_rom_type()
        else:
            rmtree(self._tmp_dir)
            raise RuntimeError(
                f"{self.input_file} is not a valid ROM file. Exiting..")

    def get_extractor(self) -> Union[LocalZipExtractor, RemoteZipExtractor]:
        """
        Get the proper extractor object according to the zip file.

        :rtype: RemoteZipExtractor or LocalZipExtractor
        """
        if "http" in self.input_file or "ota.d.miui.com" in self.input_file:
            return RemoteZipExtractor(self.input_file, self._tmp_dir)
        # elif self.input_file.endswith(".zip"):
        return LocalZipExtractor(self.input_file, self._tmp_dir)

    @staticmethod
    def get_extract_mode(extract_mode) -> ProcessTypes:
        """
        Get the extract mode enum according to extract_mode string value.

        :param extract_mode: "firmware", "nonarb", "firmwareless" or "vendor"
        :return: ProcessTypes enum
        """
        modes = {
            'firmware': ProcessTypes.firmware,
            'nonarb': ProcessTypes.non_arb_firmware,
            'firmwareless': ProcessTypes.firmware_less,
            'vendor': ProcessTypes.vendor
        }
        try:
            return modes[extract_mode]
        except KeyError as err:
            print('Unknown process!')
            raise err

    def is_valid_rom(self) -> bool:
        """
        Check if the zip file is valid ROM.

        :return: True if update-binary or updater-script is present
         in contents list, False otherwise.
        """
        return "META-INF/com/google/android/update-binary" in self.extractor.files \
               or "META-INF/com/google/android/updater-script" in self.extractor.files

    def get_rom_type(self):
        """
        Get the type of the input ROM or raise a RuntimeError.

        :return: An enum of ROM type. Either qcom or mtk.
        """
        files = str(self.extractor.files)
        if 'lk.img' in files or 'preloader.img' in files:
            self.type = ZipTypes.mtk
        elif 'firmware-update' in files or 'rpm' in files or 'tz' in files \
                or 'keymaster' in files:
            self.type = ZipTypes.qcom
        else:
            raise RuntimeError("Can't detect rom type. It's not qcom or mtk!'")

    def get_files_list(self) -> List[str]:
        """
        Get files to extract list according to the extract mode Enum.

        :return: a list of file names string.
        """
        if self.extract_mode is ProcessTypes.firmware:
            return [i for i in self.extractor.files if
                    (i.startswith('META-INF/')
                     and (i.endswith('updater-script') or i.endswith('update-binary'))
                     ) or (i.startswith('firmware-update/')
                           and 'dtbo' not in i
                           and 'splash' not in i
                           and 'logo' not in i
                           and 'vbmeta' not in i)] \
                if self.type is ZipTypes.qcom \
                else [n for n in self.extractor.files if 'system' not in n and 'vendor' not in n
                      and 'product' not in n and 'boot.img' not in n
                      and 'file_contexts' not in n]
        if self.extract_mode is ProcessTypes.non_arb_firmware:
            return [n for n in self.extractor.files if n.startswith('firmware-update/dspso.bin')
                    or n.startswith('firmware-update/BTFM.bin')
                    or n.startswith('firmware-update/NON-HLOS.bin')
                    or n.startswith('META-INF/')
                    and (n.endswith('updater-script') or n.endswith('update-binary'))]
        if self.extract_mode is ProcessTypes.firmware_less:
            return [n for n in self.extractor.files if not n.startswith('firmware-update/')] \
                if self.type is ZipTypes.qcom else []
        if self.extract_mode is ProcessTypes.vendor:
            return [n for n in self.extractor.files if not n.startswith('system')
                    and not n.startswith('vbmeta')]
        return []  # Will never happen

    def get_updater_script_lines(self) -> str:
        """
        Get selective lines from the update-script according to process type enum value.

        :return: a string of the updater-script lines
        """
        original_updater_script = Path(
            self._updater_script_dir / 'updater-script')
        if not original_updater_script.exists():
            raise FileNotFoundError("updater-script not found!")
        original_updater_script = original_updater_script.read_text().splitlines()
        lines = []
        if self.extract_mode is ProcessTypes.firmware:
            lines = [line for line in original_updater_script if "getprop" in line
                     or "Target" in line
                     or "firmware-update" in line and "dtbo.img" not in line
                     and "vbmeta" not in line and "splash" not in line
                     and "logo" not in line] \
                if self.type is ZipTypes.qcom \
                else [line for line in original_updater_script if "system" not in line
                      and "vendor" not in line and 'boot.img' not in line
                      and "dtbo.img" not in line and "vbmeta" not in line]
        elif self.extract_mode is ProcessTypes.non_arb_firmware:
            lines = [line for line in original_updater_script if "getprop" in line
                     or "Target" in line
                     or "modem" in line or "bluetooth" in line or "dsp" in line]
        elif self.extract_mode is ProcessTypes.firmware_less:
            lines = [line for line in original_updater_script if "getprop" in line
                     or "Target" in line
                     or "boot.img" in line or "system" in line or "vendor" in line
                     or "product" in line] \
                if self.type is ZipTypes.qcom else []
        elif self.extract_mode is ProcessTypes.vendor:
            lines = [line for line in original_updater_script
                     if "getprop" in line or "Target" in line
                     or "firmware-update" in line
                     or "vendor" in line] if self.type is ZipTypes.qcom \
                else [line for line in original_updater_script if
                      "system" not in line and 'boot.img' not in line]
        return '\n'.join(lines)

    def generate_updater_script(self):
        """
        Generate the new zip updater-script and write it to the temporary directory.

        :return:
        """
        current_dir = Path(__file__)
        template = Template(Path(
            current_dir.parent / 'templates/recovery_updater_script.txt').read_text())
        lines = self.get_updater_script_lines()
        if not lines:
            raise RuntimeError("Could not extract lines from updater-script!")
        if self.extract_mode is ProcessTypes.firmware:
            process = "Normal Firmware"
        elif self.extract_mode is ProcessTypes.non_arb_firmware:
            process = "non-ARB firmware"
        elif self.extract_mode is ProcessTypes.firmware_less:
            process = "firmware-less ROM"
        elif self.extract_mode is ProcessTypes.vendor:
            process = "firmware+vendor"
        else:
            process = "Unknown"  # This should never happen

        updater_script = template.substitute(datetime=self.datetime, host=self.host,
                                             process=process, lines=lines)
        # correct some updater-script lines that exist in old devices' file
        if "/firmware/image/sec.dat" in updater_script:
            updater_script = updater_script.replace(
                '/firmware/image/sec.dat', '/dev/block/bootdevice/by-name/sec')
        if '/firmware/image/splash.img' in updater_script:
            updater_script = updater_script.replace(
                '/firmware/image/splash.img', '/dev/block/bootdevice/by-name/splash')
        self.update_script = updater_script
        with open(f"{str(self._updater_script_dir)}/updater-script", "w") as out:
            out.write(updater_script)

    def make_zip(self) -> str:
        """
        Compress the temporary directory into a zip archive in output directory.

        Also, name it according to process parameters.
        :return:
        """
        out = Path(f'{self._out_dir}/result.zip')
        partial_path = '/'.join(out.parts[1:-1])
        make_archive(f"/{partial_path}/{out.stem}", 'zip', self._tmp_dir)
        if not out.exists():
            raise RuntimeError("Could not create result zip file!")
        codename = extract_codename(self.update_script)
        zip_prefix = ""
        if self.extract_mode is ProcessTypes.firmware:
            zip_prefix = "fw"
        elif self.extract_mode is ProcessTypes.non_arb_firmware:
            zip_prefix = "fw-non-arb"
        elif self.extract_mode is ProcessTypes.firmware_less:
            zip_prefix = "fw-less"
        elif self.extract_mode is ProcessTypes.vendor:
            zip_prefix = "fw-vendor"
        else:
            pass  # This should never happen
        zip_name = f"{self._out_dir}/{zip_prefix}_{codename}_{self.extractor.get_file_name()}"
        out.rename(zip_name)
        return zip_name

    def extract(self):
        """
        Invoke the extract method of extractor object.

        :return:
        """
        files_to_extract = self.get_files_list()
        if not files_to_extract or (
                len(files_to_extract) == 2
                and "updater-script" in str(files_to_extract)
                and "update-binary" in str(files_to_extract)
        ):
            self.cleanup()
            self.close()
            raise RuntimeError("Nothing found to extract!")
        self.extractor.extract(files_to_extract)

    def cleanup(self):
        """
        Cleanup temporary directory and files.

        :return:
        """
        rmtree(self._tmp_dir)

    def close(self):
        """
        Invoke close method of the extractor object.

        :return:
        """
        self.extractor.close()

    def auto(self) -> str:
        """
        Auto-pilot method for doing everything with a single call.

        This is useful for using this class from a command line interface
         or another application.
        :return: output zip file name as a string.
        """
        self.extract()
        print("Generating updater-script..")
        self.generate_updater_script()
        print("Creating new zip file..")
        new_zip = self.make_zip()
        self.cleanup()
        self.close()
        return new_zip
